<#  
.SYNOPSIS  
	Uploads multiple LPE firmware updates to all available pools.

.DESCRIPTION  
	Uploades Lync Phone Edition device firmware to all pools. It supports uploading firmware for multiple device models and multiple pools

.NOTES  
  Version      				: 1.2 - See changelog at http://www.ehloworld.com/1245 for info on what's changed
  Rights Required			: Local admin on server
  										: ExecutionPolicy of RemoteSigned or Unrestricted
  Lync Version				: 2010 (tested on CU 6)
  Author       				: Pat Richard, Exchange MVP
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Blog			: http://www.ehloworld.com/1217 
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff.
  Info Stolen from 		: 
	Acknowledgements		: Kevin Peters for some original code examples
											: Tom Arbuthot for finding a bug when running the script from ISE

.EXAMPLE
	New-CsFirmware.ps1
	
	Details
	=======
	Will prompt for a parent folder containing any firmware files, including in child folders, and will upload them to all pools that it finds.

.EXAMPLE
	New-CsFirmware.ps1 -FilesPath [path to parent folder]
	
	Details
	=======
	Will upload firmware update files within the parent folder and any child folder to all pools that it finds.

.INPUTS
	None. You cannot pipe objects to this script.

#Requires -Version 2.0
#>

[CmdletBinding(SupportsShouldProcess = $True)]
param(
	[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
	[string]$FilesPath,
	[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
	[ValidateNotNullOrEmpty()]
	[string]$LogsPath = ".\Logs",
	[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $False, Mandatory = $False)]
	[ValidateNotNullOrEmpty()]
	[string] $TranscriptName = $MyInvocation.MyCommand.Name + " {0:yyyy-MM-dd hh-mmtt}.txt" -f (Get-Date)
)

function Get-Folder {  
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[string]$message = "Select the desired folder",
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $False, Mandatory = $False)]
		[int]$path = 0
	)
  [Object]$FolderObject = New-Object -ComObject Shell.Application   
  $folder = $FolderObject.BrowseForFolder(0, $message, 0, $path)  
  if ($folder -ne $null) {  
		return $folder.self.Path  
  }else{
  	Write-Host "No folder specified"
  }
} # end function Get-Folder

function New-CsFirmware	{
	[CmdletBinding(SupportsShouldProcess = $True)]
	param()
	Set-ModuleStatus Lync
	Write-Verbose "Getting pool list"
	$FEPools = Get-CSService -UserServer
	if (!$FilesPath){
		Write-Verbose "Path not specified. Prompting for folder."
		$FilesPath = (Get-Folder "Select the folder that contains the firmware files")
	}
	Write-Verbose "Getting list of firmware update files"
	[object]$files = Get-ChildItem $FilesPath -Recurse | ? {$_.name -match "ucupdates.cab"}
	$filesfound = $files.count
	Write-Verbose "$filesfound files found"
	if ($Files -and $FEPools){
		ForEach ($File in $Files){
			ForEach ($FEPool in $FEPools) {
				Write-Verbose "Importing $file.versioninfo.filename to $FEPool.PoolFQDN"
				[string] $pool = "service:WebServer:"+$FEPool.PoolFQDN
				Import-CsDeviceUpdate -Identity $pool -FileName $file.versioninfo.filename -Verbose
			}
		}	
	}
} # end function New-CsFirmware

function Set-ModuleStatus { 
	[cmdletBinding(SupportsShouldProcess = $true)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")] 
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Write-Verbose "Importing module $name"
			Import-Module -Name "$name" 
			# module was imported
			Write-Verbose "Module $name was imported"
			# return $true
		} else {
			# module was not available
			Write-Host "Module not available." -ForegroundColor Red
			# return $false
		}
	}else {
		# module was already imported
		Write-Verbose "$name module already imported"
		# return $true
	}
} # end function Set-ModuleStatus

if (!(Test-Path $LogsPath)){New-Item $LogsPath -type Directory | Out-Null}
if (!$psise){Start-Transcript $LogsPath\$TranscriptName | Out-Null)

New-CsFirmware

if (!$psise){Stop-Transcript	| Out-Null}